<?php

namespace Drupal\book_moderation_sync\EventSubscriber;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\content_moderation\ModerationInformationInterface;
use Drupal\book\BookManagerInterface;
use Drupal\Core\Entity\EntityInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\content_moderation\Event\ContentModerationStateChangedEvent;

/**
 * Subscriber to sync moderation states between books and their children.
 */
class BookModerationSyncSubscriber implements EventSubscriberInterface {

  protected $entityTypeManager;
  protected $moderationInformation;
  protected $bookManager;

  public function __construct(EntityTypeManagerInterface $entity_type_manager, ModerationInformationInterface $moderation_information, BookManagerInterface $book_manager) {
    $this->entityTypeManager = $entity_type_manager;
    $this->moderationInformation = $moderation_information;
    $this->bookManager = $book_manager;
  }

  public static function getSubscribedEvents() {
    return [
      ContentModerationStateChangedEvent::STATE_CHANGED => 'onModerationStateChange',
    ];
  }

  public function onModerationStateChange(ContentModerationStateChangedEvent $event) {
    $entity = $event->getEntity();

    if ($this->bookManager->isBookPage($entity)) {
      $book_id = $entity->book['bid'];
      $book = $this->entityTypeManager->getStorage('node')->load($book_id);

      if ($book && $this->moderationInformation->isModeratedEntity($book)) {
        $this->syncModerationState($book, $event->getNewState());
      }
    }
  }

  protected function syncModerationState(EntityInterface $book, $state) {
    $children = $this->bookManager->getBookChildren($book->id());

    foreach ($children as $child) {
      $child_entity = $this->entityTypeManager->getStorage('node')->load($child['nid']);
      if ($child_entity && $this->moderationInformation->isModeratedEntity($child_entity)) {
        $child_entity->set('moderation_state', $state);
        $child_entity->save();
      }
    }
  }

}